import scrapy
import re
import logging
from crawl.comm.basketball import BSTMatch, BSTPrice, WLOdds, WLSpreadOdds, ScoreResult, PointsResult
from crawl.spiders.util import getNoneStr


class BasketballSpider(scrapy.Spider):
  name = 'basketball'
  custom_settings = {
    'LOG_LEVEL': logging.INFO,
    'LOG_FILE': 'logs/basketball-match.log'
  }

  # 所有比赛
  _matchesMap = {}

  def start_requests(self):
    # 受注赛事
    url = 'https://info.sporttery.cn/basketball/match_list.php'
    yield scrapy.Request(url, self.parseMatch, 'GET')

  def parseMatch(self, response):
    cssOfMatches = '.all-wrap > .match_list .m-tab tr'

    # 所有赛事
    matches = response.css(cssOfMatches)
    # 当前赛事
    for node in matches:
      tdNodeList = node.css('td')

      matchWeek = getNoneStr(tdNodeList[0].css('::text').get())  # 赛事编号
      league = getNoneStr(tdNodeList[1].css('::text').get())  # 联赛简称
      leagueFullName = getNoneStr(tdNodeList[1].attrib.get('title'))  # 联赛全称
      leagueName = '|'.join((league, leagueFullName))

      # 抓取页面上显示正确. 但是样式 zhu ke 的标记是反的
      # 暂时就不处理了。后续使用 sql 纠正过来
      homeTeam = getNoneStr(tdNodeList[2].css('.zhu::text').get())  # 主队
      awayTeam = getNoneStr(tdNodeList[2].css('.ke::text').get())  # 客队
      matchTime = getNoneStr(tdNodeList[3].css('::text').get())  # 比赛时间
      saleStatus = getNoneStr(tdNodeList[5].css('::text').get())  # 销售状态
      wdl = self.parsePassWay(tdNodeList[6].css('div'))
      wdls = self.parsePassWay(tdNodeList[7].css('div'))
      score = self.parsePassWay(tdNodeList[8].css('div'))
      points = self.parsePassWay(tdNodeList[9].css('div'))

      matchLink = tdNodeList[2].css('a').attrib.get('href')
      matchId = re.sub(r'^.*m=', '', matchLink)

      # 入库
      match = BSTMatch(
        matchWeek,
        matchTime,
        leagueName,
        homeTeam,
        awayTeam,
        saleStatus,
        wdl,
        wdls,
        score,
        points
      )
      match.persist()

      # 加入到临时字典里
      self._matchesMap[matchId] = match

      # 请求比赛赔率
      if saleStatus == '已开售':
        priceAPI = 'https://info.sporttery.cn/basketball/pool_result.php?id=' + matchId
        yield scrapy.Request(priceAPI, self.parsePrice, 'GET')

  def parsePassWay(self, div):
    # 待开售
    if div is None:
      return 'wait'

    # 地址
    cls = div.attrib.get('class')
    if cls is None:
      return 'wait'

    # 开售单关方式和过关方式
    if "u-dan" in cls:
      return 'pass&single'

    # 仅开售过关方式
    elif "u-cir" in cls:
      return 'pass'

    # 未开售此玩法
    else:
      return 'wait'

  def parsePrice(self, response):
    logging.info("采集数据源 ---> %s" % response.url)

    tableList = response.css('.kj-table')
    if tableList is None or len(tableList) < 4:
      logging.error("抓取赔率结果失败")
      return

    wl = None
    wls = None
    score = None
    points = None

    for i, tab in enumerate(tableList):
      # 取最新的记录
      tr = tab.css("tr")[-1]
      tdList = tr.css('td')

      try:
        if i == 0:
          # 胜负
          wl = WLOdds(getNoneStr(tdList[2].css("::text").get()), getNoneStr(tdList[1].css("::text").get()))
        elif i == 1:
          # 让分胜负
          wls = WLSpreadOdds(getNoneStr(tdList[3].css("::text").get()), getNoneStr(tdList[1].css("::text").get()), getNoneStr(tdList[2].css("::text").get()))
        elif i == 2:
          # 大小分
          score = ScoreResult(getNoneStr(tdList[1].css("::text").get()), getNoneStr(tdList[3].css("::text").get()), getNoneStr(tdList[2].css("::text").get()))
        elif i == 3:
          # 胜分差
          points = PointsResult(
            getNoneStr(tdList[7].css("::text").get()),
            getNoneStr(tdList[8].css("::text").get()),
            getNoneStr(tdList[9].css("::text").get()),
            getNoneStr(tdList[10].css("::text").get()),
            getNoneStr(tdList[11].css("::text").get()),
            getNoneStr(tdList[12].css("::text").get()),
            getNoneStr(tdList[1].css("::text").get()),
            getNoneStr(tdList[2].css("::text").get()),
            getNoneStr(tdList[3].css("::text").get()),
            getNoneStr(tdList[4].css("::text").get()),
            getNoneStr(tdList[5].css("::text").get()),
            getNoneStr(tdList[6].css("::text").get())
          )
        else:
          continue
      except:
        continue

    matchId = re.sub(r'^.*pool_result.php\?id=', '', response.url)
    match = self._matchesMap[matchId]

    bstprice = BSTPrice(
      matchId,
      match.matchWeek,
      match.matchTime,
      match.league,
      match.homeTeam,
      match.awayTeam,
      wl,
      wls,
      score,
      points
    )

    #
    logging.info("采集到数据 --> %s" % bstprice.toString())

    # 入库
    bstprice.persist()
